;+
; NAME:
;     MIPS_BGSUB (v0.1)
;
; PURPOSE:
;     Subtract backgrounds from MIPS fine-scale and SED data.
;
; CATEGORY:
;     MIPS (Custom)
;
; CALLING SEQUENCE:
;     MIPS_BGSUB,input_file
;
; INPUTS:
;      input_file : The name of the input FITS file
;
; KEYWORD PARAMETERS:
;       overwrite : overwrite output file
;
; OUTPUTS:
;     A FITS file with backgrounds subtracted from the on-source DCEs.
;
; MODIFICATION HISTORY:
; 	Written by: C. W. Engelbracht (2006-04-19)
;-

pro mips_bgsub,input_file,overwrite=overwrite

; check we got at least 1 parameters
if (N_params() LT 1) then begin
    print,'Syntax - mips_bgsub,input_file,overwrite=overwrite'
    return
endif

; compose output file name
len = strlen (input_file)
if (strmid (input_file, len-5, len) eq '.fits') then begin
    output_file = strmid (input_file, 0, len-5) + '.bgsub.fits'
endif else begin
    output_file = input_file + '.bgsub.fits'
endelse

; do some file checking
if (not file_test(input_file)) then begin
    print,'File (' + input_file + ') does not exist.'
    retall
endif
if (not file_test(input_file,/read)) then begin
    print,'File (' + input_file + ') exists but I cannot read it.'
    retall
endif
if (file_test(output_file)) then begin
    if (not keyword_set(overwrite)) then begin
        print,'Output file (' + output_file + ') already exists.'
        retall
    endif
    if (not file_test(output_file,/write)) then begin
        print,'Cannot overwrite (' + output_file + ').'
        retall
    endif
endif

; open files
fits_open,input_file,in_fcb
fits_open,output_file,out_fcb,/write

; read primary header and write to output
fits_read,in_fcb,image,header,exten_no=0
fits_write,out_fcb,image,header

; get CSMM positions for on- and off-source DCEs
fits_read,in_fcb,image,header,exten_no=1,/no_pdu
csm_pred_off = sxpar(header,'CSM_PRED')
fits_read,in_fcb,image,header,exten_no=3,/no_pdu
csm_pred_on = sxpar(header,'CSM_PRED')
if ((csm_pred_off eq 0) or (csm_pred_on eq 0)) then begin
    print,'Cannot read CSMM positions'
    retall
endif

; loop through extensions
for i = 1,in_fcb.nextend do begin

    ; read extension
    fits_read,in_fcb,image,header,exten_no=i,/no_pdu

    ; check CSMM position
    csm_pred = sxpar(header,'CSM_PRED')

    ; if CSMM position is close to on-source position, subtract background
    if ((abs(csm_pred-csm_pred_on)) lt (abs(csm_pred-csm_pred_off))) then begin

        ; if previous DCE was a stim, go back 1 DCE to get bkgd
        offset = 0
        if (stim_nf gt 0) then offset = 1

        ; read background DCEs
        fits_read,in_fcb,bkgd1,hdr1,exten_no=i-1-offset,/no_pdu
        fits_read,in_fcb,bkgd2,hdr2,exten_no=i+1,/no_pdu

        ; average backgrounds and subtract
        bkgd = (bkgd1 + bkgd2) / 2
        image[*,*,0] = image[*,*,0] - bkgd[*,*,0]

    endif

    ; check whether this is a stimflash
    stim_nf = sxpar(header,'STIM_NF')

    ; write extension
    fits_write,out_fcb,image,header

endfor

; close files
fits_close,in_fcb
fits_close,out_fcb

end
